import App
import loadspacehelper
import MissionLib
import Actions.MissionScriptActions
import Bridge.PowerDisplay
import Bridge.TacticalMenuHandlers
import Bridge.HelmMenuHandlers
import Tactical.Interface.TacticalControlWindow
import Bridge.BridgeUtils
import Effects
import QBGUI
import Foundation
import Lib.Ambiguity

g_version = "2.2"

qbGameMode = None 

gSystem = []
gBridge = ''
gPlayer = []
gPlayerLoc = []
gEnemy = []
gEnemyLoc = []
gEnemyStarbases = []
gEnStLoc = []
gFriendly = []
gFriendlyLoc = []
gFriendlyStarbases = []
gFrStLoc = []
gNeutral = []
gNeutralLoc = []
gNeutralStarbases = []
gNeStLoc = []
gPlugins = []
gPluginsVer = []

TRUE = 1
FALSE = 0
pEnemies = None
pFriendlies = None
pNeutrals = None
g_pDatabase = None
g_pDockButton = None
g_pMission = None

g_pSubPane = None

g_pKiska = None
g_pFelix = None
g_pSaffi = None
g_pMiguel = None
g_pBrex = None

ET_PRELOAD_DONE = None
ET_FLEETATTACK = None
ET_FLEETRESUME = None

def PreLoadAssets(pMission):
	InitLists()

	loadspacehelper.PreloadShip(gPlayer[0], 1)
	for pEnemy in gEnemy:
		loadspacehelper.PreloadShip(pEnemy[0], 1)
	for pFriendly in gFriendly:
		loadspacehelper.PreloadShip(pFriendly[0], 1)

def Initialize(pMission):
	global pEnemies
	pEnemies = pMission.GetEnemyGroup()
	global pFriendlies
	pFriendlies = pMission.GetFriendlyGroup()
	global pNeutrals
	pNeutrals = pMission.GetNeutralGroup()
	global g_pDatabase
	g_pDatabase = App.g_kLocalizationManager.Load('data/TGL/Bridge Menus.tgl')
	global g_pQBDatabase
	g_pQBDatabase = App.g_kLocalizationManager.Load('data/TGL/QuickBattle/QuickBattle.tgl')
	global g_pMission
	g_pMission = pMission
	global ET_PRELOAD_DONE
	ET_PRELOAD_DONE = App.Mission_GetNextEventType()
	global ET_FLEETATTACK
	ET_FLEETATTACK = App.Mission_GetNextEventType()
	global ET_FLEETRESUME
	ET_FLEETRESUME = App.Mission_GetNextEventType()

	global qbGameMode 
	qbGameMode = Foundation.BuildGameMode()
	qbGameMode.Activate()
	
	CreateStartingRegions()
	import LoadBridge
	LoadBridge.Load(gBridge + 'Bridge')
	InitializeCrewPointers()
	AddSystemMenus()
	CreateFleetMenu()
	#CreateObjectives()

	CreateStartingObjects(pMission)
	SetupEventHandlers(pMission)

	ConfigInit()

	executePlugins(pMission)
	
def executePlugins(pMission):
	import nt
	import string

	list = nt.listdir('scripts/Custom/QuickBattleGame/Plugins')
	list.sort()

	#dotPrefix = string.join(string.split(dir, '\\')[1:], '.') + '.'
	global gPlugins
	gPlugins = []

	for plugin in list:
		s = string.split(plugin, '.')
		if len(s) <= 1:
			continue
		extension = s[-1]
		fileName = string.join(s[:-1], '.')

		if (extension == 'pyc' or extension == 'py'):
			try:
				if (gPlugins.count(fileName) == 0):
					pModule = __import__("Custom.QuickBattleGame.plugins." + fileName)
					ver = ""
					if (dir(pModule).count('Version')>0):
						ver = pModule.Version()
					else:
						ver = fileName
					gPlugins.append(fileName)
					gPluginsVer.append(ver)
					if (dir(pModule).count('Initialize')>0):
						pModule.Initialize(pMission)
			except:
				import sys
				print sys.exc_info()[0]
				print sys.exc_info()[1]
				print sys.exc_info()[2]
	
def restartPlugins():
	for plugin in gPlugins:
		pModule = __import__("Custom.QuickBattleGame.plugins." + plugin)
		if (dir(pModule).count('Restart')>0):
			pModule.Restart()

def terminatePlugins():
	for plugin in gPlugins:
		pModule = __import__("Custom.QuickBattleGame.plugins." + plugin)
		if (dir(pModule).count('Terminate')>0):
			pModule.Terminate()
	
def Terminate(pMission):
	QBGUI.Remove()
	
	global qbGameMode
	qbGameMode.Deactivate()
	qbGameMode = None
	
	RemoveInstanceHandlers()

	if (g_pDatabase != None):
		App.g_kLocalizationManager.Unload(g_pDatabase)
		
	terminatePlugins()
	
	Lib.Ambiguity.terminate()

def InitLists():
	global gSystem, gBridge, gPlayer, gPlayerLoc, gEnemy, gEnemyLoc
	global gEnemyStarbases, gEnStLoc, gFriendly, gFriendlyLoc, gFriendlyStarbases
	global gFrStLoc, gNeutral, gNeutralLoc, gNeutralStarbases, gNeStLoc
	global g_pMessageDatabase

	import QBSetup
	gSystem                =  QBSetup.gSystem           
	gBridge                =  QBSetup.gBridge           
	gPlayer                =  QBSetup.gPlayer           
	gPlayerLoc             =  QBSetup.gPlayerLoc        
	gEnemy                 =  QBSetup.gEnemy            
	gEnemyLoc              =  QBSetup.gEnemyLoc
	gEnemyStarbases        =  QBSetup.gEnemyStarbases   
	gEnStLoc               =  QBSetup.gEnStLoc          
	gFriendly              =  QBSetup.gFriendly         
	gFriendlyLoc           =  QBSetup.gFriendlyLoc      
	gFriendlyStarbases     =  QBSetup.gFriendlyStarbases
	gFrStLoc               =  QBSetup.gFrStLoc          
	gNeutral               =  QBSetup.gNeutral         
	gNeutralLoc            =  QBSetup.gNeutralLoc      
	gNeutralStarbases      =  QBSetup.gNeutralStarbases
	gNeStLoc               =  QBSetup.gNeStLoc
	
	try:
		import plugin
		plugin.gPluginData = QBSetup.gPluginData
	except:
		plugin.gPluginData = {}

def InitializeCrewPointers():
	pBridge = App.g_kSetManager.GetSet('bridge')
	global g_pKiska
	g_pKiska = App.CharacterClass_GetObject(pBridge, 'Helm')
	global g_pFelix
	g_pFelix = App.CharacterClass_GetObject(pBridge, 'Tactical')
	global g_pSaffi
	g_pSaffi = App.CharacterClass_GetObject(pBridge, 'XO')
	global g_pMiguel
	g_pMiguel = App.CharacterClass_GetObject(pBridge, 'Science')
	global g_pBrex
	g_pBrex = App.CharacterClass_GetObject(pBridge, 'Engineer')
	
def AddSystemMenus():
	for system in gSystem:
		if (system[0] == 'Starbase12'):
			import Systems.Starbase12.Starbase
			pMenu = Systems.Starbase12.Starbase.CreateMenus()
		elif (system[0] == 'DryDock'):
			import Systems.DryDock.DryDockSystem
			pMenu = Systems.DryDock.DryDockSystem.CreateMenus()		
		elif (system[0] == 'QuickBattle'):
			import Systems.QuickBattle.QuickBattleSystem
			pMenu = Systems.QuickBattle.QuickBattleSystem.CreateMenus()		
		else:
			pModule = __import__('Systems.' + system[0] + '.' + system[0])
			if (hasattr(pModule, "CreateMenus")):
				pMenu = pModule.CreateMenus()
			else:
				import Systems.Utils
				Systems.Utils.CreateSystemMenu(system[0], 'Systems.' + system[0] + '.' + system[0])

		if (system[2] != ''):
			pMenu.SetEpisodeName(system[2])
		
def RemoveSystemMenus():
	App.SortedRegionMenu_ClearSetCourseMenu()		

def CreateStartingObjects(pMission):
	pQBSet = App.g_kSetManager.GetSet(gPlayer[2])
	
	pPlayer = MissionLib.CreatePlayerShip(gPlayer[0], pQBSet, gPlayer[1], '')
	TranslateShip(pPlayer, gPlayerLoc[0], gPlayerLoc[1], gPlayerLoc[2])
	if (len(gPlayer)>3):
		DamageShipRandomly(pPlayer, gPlayer[3][0], gPlayer[3][1])

	
	AddShips();
	
	pPlayer = MissionLib.GetPlayer()
	if (pPlayer is None):
		return 

def AddShips():	
	import QuickBattleAI
	import QuickBattleFriendlyAI
	import StarbaseAI
	import StarbaseFriendlyAI
	
	pEnemies.RemoveAllNames()
	pFriendlies.RemoveAllNames()
	pNeutrals.RemoveAllNames()

	for pEnemy in gEnemy:
		pEnemies.AddName(pEnemy[1])
	for pEnemy in gEnemyStarbases:
		pEnemies.AddName(pEnemy[1])

        pFriendlies.AddName(gPlayer[1])
	for pFriend in gFriendly:
		pFriendlies.AddName(pFriend[1])
		MissionLib.AddCommandableShip(pFriend[1])
	for pFriend in gFriendlyStarbases:
		pFriendlies.AddName(pFriend[1])

	for pNeutral in (gNeutral+gNeutralStarbases):
		pNeutrals.AddName(pNeutral[1])


	i = 0
	for pEnemy in gEnemy:
		pSet = App.g_kSetManager.GetSet(pEnemy[2])
		pEnemyShip = loadspacehelper.CreateShip(pEnemy[0], pSet, pEnemy[1], '')
		TranslateShip(pEnemyShip, gEnemyLoc[i][0], gEnemyLoc[i][1], gEnemyLoc[i][2])
		if (pEnemy[4] == ''):
			pEnemyShip.SetAI(QuickBattleAI.CreateAI(pEnemyShip, pEnemy[5], pEnemy[3]))
		else:
			pModule = __import__(pEnemy[4])
			ai = apply(pModule.CreateAI, (pEnemyShip,))
			pEnemyShip.SetAI(ai)
		if (len(pEnemy)>6):
			DamageShipRandomly(pEnemyShip, pEnemy[6][0], pEnemy[6][1])
			
		i = i + 1
	i = 0
	for pEnemy in gEnemyStarbases:
		pSet = App.g_kSetManager.GetSet(pEnemy[2])
		pEnemyShip = loadspacehelper.CreateShip(pEnemy[0], pSet, pEnemy[1], '')
		TranslateShip(pEnemyShip, gEnStLoc[i][0], gEnStLoc[i][1], gEnStLoc[i][2])
		if (pEnemy[3] == ''):
			pEnemyShip.SetAI(StarbaseAI.CreateAI(pEnemyShip))
		else:
			pModule = __import__(pEnemy[3])
			ai = apply(pModule.CreateAI, (pEnemyShip,))
			pEnemyShip.SetAI(ai)
		if (len(pEnemy)>4):
			DamageShipRandomly(pEnemyShip, pEnemy[4][0], pEnemy[4][1])
		i = i + 1
	
	i = 0
	for pFriend in gFriendly:
		pSet = App.g_kSetManager.GetSet(pFriend[2])
		pFriendlyShip = loadspacehelper.CreateShip(pFriend[0], pSet, pFriend[1], '')
		TranslateShip(pFriendlyShip, gFriendlyLoc[i][0], gFriendlyLoc[i][1], gFriendlyLoc[i][2])
		if (pFriend[4] == ''):
			pFriendlyShip.SetAI(QuickBattleFriendlyAI.CreateAI(pFriendlyShip, pFriend[5], pFriend[3]))
		else:
			pModule = __import__(pFriend[4])
			ai = apply(pModule.CreateAI, (pFriendlyShip,))
			pFriendlyShip.SetAI(ai)
		if (len(pFriend)>6):
			DamageShipRandomly(pFriendlyShip, pFriend[6][0], pFriend[6][1])
		i = i + 1
	i = 0
	for pFriend in gFriendlyStarbases:
		pSet = App.g_kSetManager.GetSet(pFriend[2])
		pFriendlyShip = loadspacehelper.CreateShip(pFriend[0], pSet, pFriend[1], '')
		TranslateShip(pFriendlyShip, gFrStLoc[i][0], gFrStLoc[i][1], gFrStLoc[i][2])
		if (pFriend[3] == ''):
			pFriendlyShip.SetAI(StarbaseFriendlyAI.CreateAI(pFriendlyShip))
		else:
			pModule = __import__(pFriend[3])
			ai = apply(pModule.CreateAI, (pFriendlyShip,))
			pFriendlyShip.SetAI(ai)
		if (len(pFriend)>4):
			DamageShipRandomly(pFriendlyShip, pFriend[4][0], pFriend[4][1])
		i = i + 1

	i = 0
	for pNeutral in gNeutral:
		pSet = App.g_kSetManager.GetSet(pNeutral[2])
		pNeutralShip = loadspacehelper.CreateShip(pNeutral[0], pSet, pNeutral[1], '')
		TranslateShip(pNeutralShip, gNeutralLoc[i][0], gNeutralLoc[i][1], gNeutralLoc[i][2])
		if (pNeutral[4] != ''):
			pModule = __import__(pNeutral[4])
			ai = apply(pModule.CreateAI, (pNeutralShip,))
			pNeutralShip.SetAI(ai)
		if (len(pNeutral)>6):
			DamageShipRandomly(pNeutralShip, pNeutral[6][0], pNeutral[6][1])			
		i = i + 1
	i = 0
	
	for pNeutral in gNeutralStarbases:
		pSet = App.g_kSetManager.GetSet(pNeutral[2])
		pNeutralShip = loadspacehelper.CreateShip(pNeutral[0], pSet, pNeutral[1], '')
		TranslateShip(pNeutralShip, gNeStLoc[i][0], gNeStLoc[i][1], gNeStLoc[i][2])
		if (pNeutral[3] != ''):
			pModule = __import__(pNeutral[3])
			ai = apply(pModule.CreateAI, (pNeutralShip,))
			pNeutralShip.SetAI(ai)
		if (len(pNeutral)>4):
			DamageShipRandomly(pNeutralShip, pNeutral[4][0], pNeutral[4][1])
		i = i + 1


def CreateObjectives():
	pBridgeSet = App.g_kSetManager.GetSet('bridge')
	pSaffi = App.CharacterClass_GetObject(pBridgeSet, 'XO')


	pSaffiMenu = g_pSaffi.GetMenu()
	if (pSaffiMenu == None):
		return 0
	pObjectivesMenu = pSaffiMenu.GetSubmenuW(g_pDatabase.GetString('Objectives'))
	if (pObjectivesMenu != None):
		pass


def TranslateShip(pShip, x, y, z):
	pPlayer = App.Game_GetCurrentPlayer()

	if (x == y == z == 0):
		if (pShip.GetName() != pPlayer.GetName()):
			PlaceShipRandomly(pShip)
	else:
		p = App.TGPoint3()
		p.SetX(x)
		p.SetY(y)
		p.SetZ(z)
		pShip.SetTranslate(p)


def CreateStartingRegions():
	for system in gSystem:
		if (App.g_kSetManager.GetSet(system[1]) is None):
			MissionLib.SetupSpaceSet('Systems.' + system[0] + '.' + system[1])
	
def CreateFleetMenu():
	pHailMenu = MissionLib.GetCharacterSubmenu("Helm", "Hail")

	pFleetMenu = App.STMenu_CreateW(App.TGString("Fleet")) 
	pHailMenu.AddChild(pFleetMenu)

	g_pKiska.AddPythonFuncHandlerForInstance(ET_FLEETATTACK, (__name__ + '.FleetAttack'))
	pFleetMenu.AddChild(CreateBridgeMenuButton(App.TGString('Attack Target'), ET_FLEETATTACK, 0, g_pKiska))
	g_pKiska.AddPythonFuncHandlerForInstance(ET_FLEETRESUME, (__name__ + '.FleetResume'))
	pFleetMenu.AddChild(CreateBridgeMenuButton(App.TGString('Resume Old Orders'), ET_FLEETRESUME, 0, g_pKiska))

def CreateBridgeMenuButton(pName, eType, iSubType, pCharacter):
	pEvent = App.TGIntEvent_Create()
	pEvent.SetEventType(eType)
	pEvent.SetDestination(pCharacter)
	pEvent.SetInt(iSubType)
	BridgeMenuButton = App.STButton_CreateW(pName, pEvent)
	return BridgeMenuButton
	
def SetupEventHandlers(pMission):
	App.g_kEventManager.AddBroadcastPythonFuncHandler(App.ET_OBJECT_DESTROYED, pMission, __name__ + ".ShipDestroyed")
	App.g_kEventManager.AddBroadcastPythonFuncHandler(App.ET_OBJECT_EXPLODING, pMission, __name__ + ".ShipExploding")

	pKiskaMenu = Bridge.BridgeUtils.GetBridgeMenu("Helm")
	if (pKiskaMenu != None):
		pKiskaMenu.AddPythonFuncHandlerForInstance(App.ET_DOCK,	__name__ + ".DockButtonClicked")
		pKiskaMenu.AddPythonFuncHandlerForInstance(App.ET_HAIL,	__name__ + ".HailHandler")

	pBrexMenu = Bridge.BridgeUtils.GetBridgeMenu('Engineer')
	if (pBrexMenu != None):
		event = Lib.Ambiguity.getEvent("ET_TRANSPORT")
		g_pBrex.AddPythonFuncHandlerForInstance(event, __name__ + ".Transport")
		pButton = CreateBridgeMenuButton(App.TGString('Transporter'), event, 0, g_pBrex)
		pBrexMenu.PrependChild(pButton)

	
	event = Lib.Ambiguity.addEventHandler("XO", "ET_CONFIG", __name__ + ".Config")
	Lib.Ambiguity.createMenuButton("XO", "Configure", event, "Configure")
	event = Lib.Ambiguity.addEventHandler("XO", "ET_CREDITS", __name__ + ".Credits")
	Lib.Ambiguity.createMenuButton("XO", "Credits", event, "Credits")

		

def RemoveInstanceHandlers():
	pBrexMenu = Bridge.BridgeUtils.GetBridgeMenu('Engineer')
	if (pBrexMenu != None):
		g_pBrex.RemoveHandlerForInstance(Lib.Ambiguity.getEvent("ET_TRANSPORT"), __name__ + ".Transport")
		
	pKiskaMenu = Bridge.BridgeUtils.GetBridgeMenu("Helm")
	if (not pKiskaMenu is None):
		pKiskaMenu.RemoveHandlerForInstance(App.ET_DOCK, __name__ + ".DockButtonClicked")
		pKiskaMenu.RemoveHandlerForInstance(App.ET_HAIL, __name__ + ".HailHandler")
		g_pKiska.RemoveHandlerForInstance(ET_FLEETATTACK, (__name__ + '.FleetAttack'))
		g_pKiska.RemoveHandlerForInstance(ET_FLEETRESUME, (__name__ + '.FleetResume'))


def DockButtonClicked(pObject, pEvent):	
	pPlayer = App.Game_GetCurrentPlayer()
	if pPlayer:
		DockStarbase()

	pObject.CallNextHandler(pEvent)
	import BridgeHandlers
	BridgeHandlers.DropMenusTurnBack()

def DockStarbase():
	# Get Player.
	pPlayer = MissionLib.GetPlayer()
	pStarbase = FindStarbase(pPlayer)
	
	if (pStarbase is None):
		return

	MissionLib.SetTotalTorpsAtStarbase("Photon", -1)
	MissionLib.SetTotalTorpsAtStarbase("Quantum", -1)
	MissionLib.SetTotalTorpsAtStarbase("Adv. Photon", -1)
	MissionLib.SetTotalTorpsAtStarbase("Positron", -1)
	MissionLib.SetTotalTorpsAtStarbase("Phased", -1)

	pGraffAction = None
	# Set AI for docking/undocking.
	import AI.Compound.DockWithStarbase
	MissionLib.SetPlayerAI("Helm", AI.Compound.DockWithStarbase.CreateAI(pPlayer, pStarbase, pGraffAction, NoRepair = FALSE, FadeEnd = TRUE))

def FleetDock(pShip):	
	pStarbase = FindStarbase(pShip)
			
	if(pStarbase is None):
		return

	MissionLib.SetTotalTorpsAtStarbase("Photon", -1)
	MissionLib.SetTotalTorpsAtStarbase("Quantum", -1)
	MissionLib.SetTotalTorpsAtStarbase("Adv. Photon", -1)
	MissionLib.SetTotalTorpsAtStarbase("Positron", -1)
	MissionLib.SetTotalTorpsAtStarbase("Phased", -1)
	Bridge.HelmMenuHandlers.OverrideAI(pShip, "AI.Fleet.DockStarbase", pShip.GetObjID(), pStarbase.GetObjID(), None, NoRepair = FALSE, FadeEnd = TRUE)


def FindStarbase(pShip):
	if pShip:
		pStarbaseSet = pShip.GetContainingSet()
		if(pStarbaseSet is None):
			return
	
		# Get Starbase.
		pStarbase = None
		for pFriend in gFriendlyStarbases:
			pStarbase = pStarbaseSet.GetObject(pFriend[1])
			if ((not pStarbase is None) or (pFriend[0] == "FedStarbase")):
				break
			
		
		if(pStarbase is None):
			return
		
		return pStarbase

def FleetAttack(pObject, pEvent):
	pPlayer = MissionLib.GetPlayer()
	pTarget = pPlayer.GetTarget()
	pSet = pPlayer.GetContainingSet()
	if pTarget:
		lpFriendlies = pFriendlies.GetActiveObjectTupleInSet(pSet)
		for pObject in lpFriendlies:
			pFriend = App.ShipClass_Cast(pObject)
			if (pFriend.GetName() != pPlayer.GetName()) and (pFriend.GetImpulseEngineSubsystem() != None):
				import Bridge.HelmMenuHandlers
				Bridge.HelmMenuHandlers.OverrideAI(pFriend, "AI.Fleet.DestroyTarget", pFriend.GetObjID(), pTarget.GetObjID())

	if (pObject):
		pObject.CallNextHandler(pEvent)

def FleetResume(pObject, pEvent):
	pPlayer = MissionLib.GetPlayer()
	pTarget = pPlayer.GetTarget()
	pSet = pPlayer.GetContainingSet()
	if pTarget:
		lpFriendlies = pFriendlies.GetActiveObjectTupleInSet(pSet)
		for pObject in lpFriendlies:
			pFriend = App.ShipClass_Cast(pObject)
			import Bridge.HelmMenuHandlers
			Bridge.HelmMenuHandlers.StopOverridingAI(pFriend)
	
	if (pObject):
		pObject.CallNextHandler(pEvent)

def HailHandler(TGObject, pEvent):
	# Get the player and their target
	pPlayer = MissionLib.GetPlayer()
	if (pPlayer is None):
		return
	pTarget	= App.ObjectClass_Cast(pEvent.GetSource())
	if (pTarget is None):
		return
		
	sTargetName = pTarget.GetName()

	pStarbase = FindStarbase(pPlayer)
	if (pStarbase is None):
		return
		
	sStarbaseName = pStarbase.GetName()
	
	if (sTargetName == sStarbaseName):
		DockStarbase()
	else:
		FleetDock(pTarget)

	
def Credits(pObject, pEvent):

	pPane = App.TGPane_Create(1.0, 1.0)
	App.g_kRootWindow.PrependChild(pPane)	# stick it in front

	MissionLib.LookForward(None, TRUE)

	pSequence = App.TGSequence_Create()
	pSequence.SetUseRealTime (1)
	pSequence.SetSkippable(1)

	pAction = LineAction("QuickBattle Replacement " + g_version, pPane, 15, 5, 16)
	pSequence.AddAction(pAction, None, 0)

	pAction = LineAction("Created 2002 by Banbury", pPane, 17, 5, 12)
	pSequence.AddAction(pAction, None, 1)
	
	pAction = LineAction("Tutorial and additional changes by", pPane, 15, 5, 14)
	pSequence.AddAction(pAction, None, 7)
	pAction = LineAction("Capt. Redbeard", pPane, 18, 6, 16)
	pSequence.AddAction(pAction, None, 7)

	pAction = LineAction("Greetings to", pPane, 12, 8, 14)
	pSequence.AddAction(pAction, None, 16)
	pAction = LineAction("all the nice people", pPane, 15, 8, 12)
	pSequence.AddAction(pAction, None, 16)
	pAction = LineAction("at Bridge Commander Network", pPane, 17, 8, 12)
	pSequence.AddAction(pAction, None, 16)
	
	pAction = LineAction("Special Greetings to", pPane, 15, 5, 14)
	pSequence.AddAction(pAction, None, 25)
	pAction = LineAction("Dasher42", pPane, 18, 6, 16)
	pSequence.AddAction(pAction, None, 26)

	pSequence.Play()
	
	return 0

def LineAction(sLine, pPane, fPos, duration, fontSize):
	fHeight = fPos * 0.0375
	pAction = App.TGCreditAction_CreateSTR(sLine, pPane, 0.0, fHeight, duration, 0.25, 0.5, fontSize)
	return pAction
	
def ConfigInit():
	QBGUI.SetBridge(gBridge)

	QBGUI.Init(g_pMission)
	QBGUI.g_pShipListData.clear()
	
	data = QBGUI.GetDefaultShipData(gPlayer[0])
	data['name'] = gPlayer[1]
	data['system'] = gPlayer[2]
	data['pos'] = gPlayerLoc
	data['ailevel'] = 0.0
	data['ai'] = ''
	data['warp'] = 0
	data['group'] = 'player'
	if (len(gPlayer)>3):
		data['mindamage'] = gPlayer[3][0]
		data['maxdamage'] = gPlayer[3][1]
	QBGUI.AddShip(gPlayer[0], data)

	QBGUI.AddGroup(gFriendly, gFriendlyLoc, 'friend', 0)
	QBGUI.AddGroup(gFriendlyStarbases, gFrStLoc, 'friend', 1)
	QBGUI.AddGroup(gEnemy, gEnemyLoc, 'enemy', 0)
	QBGUI.AddGroup(gEnemyStarbases, gEnStLoc, 'enemy', 1)
	QBGUI.AddGroup(gNeutral, gNeutralLoc, 'neutral', 0)
	QBGUI.AddGroup(gNeutralStarbases, gNeStLoc, 'neutral', 1)
	
	for system in gSystem:
		QBGUI.AddSystem(system[0]+'.'+system[1])
		
	g_pSaffi.AddPythonFuncHandlerForInstance(QBGUI.ET_START_BUTTON, (__name__ + '.StartSimulation'))



def Config(pObject, pEvent):
	pTopWindow = App.TopWindow_GetTopWindow()
	if (not pTopWindow.IsBridgeVisible()):
		pTopWindow.ForceBridgeVisible()
	
#	pSequence = App.TGSequence_Create()
#	pTest = App.CharacterAction_Create(g_pSaffi, App.CharacterAction.AT_PLAY_ANIMATION_FILE, "skel1_3")
#	pTest = App.CharacterAction_Create(g_pSaffi, App.CharacterAction.AT_PLAY_ANIMATION_FILE, "pointing_left")
#	pSequence.AppendAction(pTest)
#	pSequence.Play()

	QBGUI.OpenDialog()
	
	if (pObject):
		pObject.CallNextHandler(pEvent)

def StartSimulation(pObject, pEvent):
	if (QBGUI.CheckSetup() == 0):
		pTopWindow = App.TopWindow_GetTopWindow()
		pModalDialogWindow = App.ModalDialogWindow_Cast (pTopWindow.FindMainWindow (App.MWT_MODAL_DIALOG))
		pModalDialogWindow.Run(None, App.TGString("Not enough data. Please check that all your ships have a name and a starting system and that you have assigned a PLAYER ship."), App.TGString("OK"), None, None, None)
	else:
		ResetQB()
		if (pObject):
			pObject.CallNextHandler(pEvent)

def ChangeGlobals():
	global gBridge
	gBridge = QBGUI.g_Bridge
	
	global gSystem
	gSystem = []
	for system in QBGUI.g_pRegionListData:
		import strop
		s = strop.split(system, '.')
		gSystem.append([s[0], s[1], ''])
		
	global gPlayer
	global gPlayerLoc
	global gEnemy
	gEnemy = []
	global gEnemyLoc
	gEnemyLoc = []
	global gEnemyStarbases
	gEnemyStarbases = []
	global gEnStLoc
	gEnStLoc = []
	global gFriendly
	gFriendly = []
	global gFriendlyLoc
	gFriendlyLoc = []
	global gFriendlyStarbases
	gFriendlyStarbases = []
	global gFrStLoc
	gFrStLoc = []
	global gNeutral
	gNeutral = []
	global gNeutralLoc
	gNeutralLoc = []
	global gNeutralStarbases
	gNeutralStarbases = []
	global gNeStLoc
	gNeStLoc = []
	
	keys = QBGUI.g_pShipListData.keys()
	for key in keys:
		ship = QBGUI.g_pShipListData[key]
		if (ship['group']=='player'):
			gPlayer = [ship['class'], ship['name'], ship['system'], [ship['mindamage'], ship['maxdamage']]]
			gPlayerLoc = ship['pos']
		elif (ship['group']=='enemy'):
			if (not ship['starbase']):
				gEnemy.append([ship['class'], ship['name'], ship['system'], ship['warp'], ship['ai'], ship['ailevel'], [ship['mindamage'], ship['maxdamage']]])
				gEnemyLoc.append(ship['pos'])
			else:
				gEnemyStarbases.append([ship['class'], ship['name'], ship['system'],  ship['ai'], [ship['mindamage'], ship['maxdamage']]])
				gEnStLoc.append(ship['pos'])
		elif (ship['group']=='friend'):
			if (not ship['starbase']):
				gFriendly.append([ship['class'], ship['name'], ship['system'], ship['warp'], ship['ai'], ship['ailevel'], [ship['mindamage'], ship['maxdamage']]])
				gFriendlyLoc.append(ship['pos'])
			else:
				gFriendlyStarbases.append([ship['class'], ship['name'], ship['system'],  ship['ai'], [ship['mindamage'], ship['maxdamage']]])
				gFrStLoc.append(ship['pos'])
		elif (ship['group']=='neutral'):
			if (not ship['starbase']):
				gNeutral.append([ship['class'], ship['name'], ship['system'], ship['warp'], ship['ai'], ship['ailevel'], [ship['mindamage'], ship['maxdamage']]])
				gNeutralLoc.append(ship['pos'])
			else:
				gNeutralStarbases.append([ship['class'], ship['name'], ship['system'],  ship['ai'], [ship['mindamage'], ship['maxdamage']]])
				gNeStLoc.append(ship['pos'])


def ResetQB():
	pTopWindow = App.TopWindow_GetTopWindow()
	pTopWindow.AllowKeyboardInput(0)
	pTopWindow.AllowMouseInput(0)

	Bridge.TacticalMenuHandlers.ClearOrderMenus ()
	App.CharacterAction_Create(g_pSaffi, App.CharacterAction.AT_MENU_DOWN).Play()
	RemoveSystemMenus()

	for pSet in App.g_kSetManager.GetAllSets():	
		for kShip in pSet.GetClassObjectList(App.CT_DAMAGEABLE_OBJECT):
			if kShip.GetName() != MissionLib.GetPlayer().GetName():
				pSet.RemoveObjectFromSet(kShip.GetName())
				kShip.SetDeleteMe(1)
		for kTorp in pSet.GetClassObjectList(App.CT_TORPEDO):
				kTorp.SetDeleteMe(1)

	ChangeGlobals()
	CreateStartingRegions()
	RecreatePlayer()
	AddSystemMenus()
	AddShips()

	# Switch us to green alert
	pAlertEvent = App.TGIntEvent_Create()
	pAlertEvent.SetDestination(App.Game_GetCurrentGame().GetPlayer())
	pAlertEvent.SetEventType(App.ET_SET_ALERT_LEVEL)
	pAlertEvent.SetInt(App.Game_GetCurrentGame().GetPlayer().GREEN_ALERT)
	App.g_kEventManager.AddEvent(pAlertEvent)

	# dump the unused models
	App.g_kLODModelManager.Purge()

	pTopWindow.AllowKeyboardInput(1)
	pTopWindow.AllowMouseInput(1)
	
	restartPlugins()
		
def RecreatePlayer():
	pPlayer = MissionLib.GetPlayer()
	pSet = None
	if pPlayer:
		pSet = pPlayer.GetContainingSet()
		pPlayer.SetDead()
		pSet.DeleteObjectFromSet(pPlayer.GetName())

	pQBSet = App.g_kSetManager.GetSet(gPlayer[2])

	pPlayer = MissionLib.CreatePlayerShip(gPlayer[0], pQBSet, gPlayer[1], '')
	TranslateShip(pPlayer, gPlayerLoc[0], gPlayerLoc[1], gPlayerLoc[2])

	# Update the ship with its new positional information...
	pPlayer.UpdateNodeOnly()

	# update the proximity manager with this object's new position.
	pProximityManager = pQBSet.GetProximityManager()
	if (pProximityManager):
		pProximityManager.UpdateObject (pPlayer)

	# Reload our bridge, just in case
	import LoadBridge
	LoadBridge.Load(gBridge+"Bridge")

#	path = "data/Textures/Uniforms/voyager/"
#	import QBCharacter
#	global g_pMiguel
#	g_pMiguel = QBCharacter.LoadCrew(path, gBridge)


	# Reassign global pointers
	InitializeCrewPointers()

	return pPlayer
	
def PlaceShipRandomly(pShip):
	pSet = pShip.GetContainingSet()
	
	vLocation = pShip.GetWorldLocation()
	vForward = pShip.GetWorldForwardTG()
	vForward.Scale(pShip.GetRadius() + pShip.GetRadius() + 200.0)

	kPoint = App.TGPoint3()
	ChooseNewLocation(vLocation, vForward)
	kPoint.Set(vLocation)
	kPoint.Add(vForward)

	fRadius = pShip.GetRadius () * 2.0

	while pSet.IsLocationEmptyTG(kPoint, fRadius, 1) == 0:
		ChooseNewLocation(vLocation, vForward)
		kPoint.Set(vLocation)
		kPoint.Add(vForward)

	pShip.SetTranslate(kPoint)

def ChooseNewLocation(vOrigin, vOffset):
	# Add some random amount to vOffset
	fUnitRandom = (App.g_kSystemWrapper.GetRandomNumber(10001) - 5000.0) / 5000.0
	fUnitRandom = fUnitRandom * (App.g_kSystemWrapper.GetRandomNumber (1000) + 750)

	vOffset.SetX( vOffset.GetX() + fUnitRandom )


	fUnitRandom = (App.g_kSystemWrapper.GetRandomNumber(10001) - 5000.0) / 5000.0
	fUnitRandom = fUnitRandom * (App.g_kSystemWrapper.GetRandomNumber (1000) + 750)

	vOffset.SetY( vOffset.GetY() + fUnitRandom )

	fUnitRandom = (App.g_kSystemWrapper.GetRandomNumber(10001) - 5000.0) / 5000.0
	fUnitRandom = fUnitRandom * (App.g_kSystemWrapper.GetRandomNumber (1000) + 750)

	vOffset.SetZ( vOffset.GetZ() + fUnitRandom )

	return 0

def ShipDestroyed(pObject, pEvent):
	pPlayer = App.Game_GetCurrentGame().GetPlayer()

	# Get the ship that was destroyed.
	pShip = App.ShipClass_Cast(pEvent.GetDestination())
	if (pShip != None):
		if (pPlayer and pShip.GetName() == pPlayer.GetName()):
			ResetQB()
			# Force back to the bridge
			pTopWindow = App.TopWindow_GetTopWindow()
			pTopWindow.ForceBridgeVisible()

			# Need to reset interactive mode, otherwise you'll get stuck if you go
			# to this mode.
			pCinematic = App.CinematicWindow_Cast(pTopWindow.FindMainWindow(App.MWT_CINEMATIC))
			if pCinematic:
				pCinematic.SetInteractive(1)

			if pPlayer:
				pPlayer.SetTarget(None)
		else:
			if (pEnemies.IsNameInGroup(pShip.GetName())):
				pLine1 = App.CharacterAction_Create(g_pFelix, App.CharacterAction.AT_SAY_LINE, "QBEnemyGenericShipDestroyed", "Captain", 1, g_pDatabase)
			elif (pFriendlies.IsNameInGroup(pShip.GetName())):
				pLine1 = App.CharacterAction_Create(g_pFelix, App.CharacterAction.AT_SAY_LINE, "QBFriendlyGenericShipDestroyed", "Captain", 1, g_pDatabase)
			else:
				pLine1 = App.CharacterAction_Create(g_pFelix, App.CharacterAction.AT_SAY_LINE, "QBFriendlyGenericShipDestroyed", "Captain", 1, g_pDatabase)
	
			pSequence = App.TGSequence_Create()
			pSequence.AddAction(pLine1)
			pSequence.Play()
		
	if (pObject):
		pObject.CallNextHandler(pEvent)
	
def ShipExploding(pObject, pEvent):
	pPlayer = App.Game_GetCurrentGame().GetPlayer()

	# Get the ship that was destroyed.
	pShip = App.ShipClass_Cast(pEvent.GetDestination())
	if (pShip != None):
		if (pPlayer and pShip.GetName() == pPlayer.GetName()):
			import DynamicMusic
			DynamicMusic.PlayFanfare("PlayerBlewUp")
		else:
			import DynamicMusic
			DynamicMusic.PlayFanfare("EnemyBlewUp")
			
	if (pObject):
		pObject.CallNextHandler(pEvent)

def Distance(pObject):
	pPlayer = App.Game_GetCurrentGame().GetPlayer()
	vDifference = pObject.GetWorldLocation()
	vDifference.Subtract(pPlayer.GetWorldLocation())

	return vDifference.Length()

def Transport(pObject, pEvent):
	pPlayer = MissionLib.GetPlayer()
	pTarget = pPlayer.GetTarget()
	pSet = pPlayer.GetContainingSet()
	dist = 300
	pClosest = None
	lpFriendlies = pFriendlies.GetActiveObjectTupleInSet(pSet)
	for pObject in lpFriendlies:
		pFriend = App.ShipClass_Cast(pObject)
		d = Distance(pFriend)
		if (d<dist) and (pFriend.GetName() != pPlayer.GetName()) and pFriend.GetImpulseEngineSubsystem() != None:
			pClosest = pFriend
			dist = d
	
	if (pClosest != None):
		MissionLib.RemoveCommandableShip(pClosest.GetName())
		MissionLib.AddCommandableShip(pPlayer.GetName())
		ai = pClosest.GetAI()

		pGame = App.Game_GetCurrentGame()

		pSetPlayerAction = App.TGScriptAction_Create(__name__, "SetPlayerAction", pClosest)
		
		pSequence = App.TGSequence_Create()
		
		pSequence.AppendAction(App.TGScriptAction_Create("Actions.MissionScriptActions", "ChangeToBridge"))
		pSequence.AppendAction(App.TGScriptAction_Create("MissionLib", "StartCutscene"))
		pSequence.AppendAction(App.TGScriptAction_Create("MissionLib", "SetTarget", pClosest.GetName()))
		pSequence.AppendAction(App.TGScriptAction_Create("Actions.CameraScriptActions", "CutsceneCameraBegin", pSet.GetName()), 2)
		pSequence.AppendAction(App.TGScriptAction_Create("Actions.CameraScriptActions", "ChangeRenderedSet", pSet.GetName()))
		pSequence.AppendAction(pSetPlayerAction)	
		pSequence.AppendAction(App.TGScriptAction_Create("Actions.CameraScriptActions", "LockedView", pSet.GetName(), pClosest.GetName(), 30, 30, 15))
		
		pSequence.AppendAction(App.TGScriptAction_Create("Actions.CameraScriptActions", "CutsceneCameraEnd", pSet.GetName()), 6)	
		pSequence.AppendAction(App.TGScriptAction_Create("Actions.CameraScriptActions", "ChangeRenderedSet", "bridge"))
		pSequence.AppendAction(App.TGScriptAction_Create("MissionLib", "SetTarget", pPlayer.GetName()))
		pSequence.AppendAction(App.TGScriptAction_Create("MissionLib", "EndCutscene"))

		pSequence.Play()

		MissionLib.SetPlayerAI("Captain", None)
		pPlayer.ClearAI()

	if (pObject):
		pObject.CallNextHandler(pEvent)

def SetPlayerAction(pAction, pShip):
	pGame = App.Game_GetCurrentGame()
	pGame.SetPlayer(pShip)
	return 0
		
def ZoomTarget(pAction, pSource, pTarget):
	import Camera
	Camera.ZoomTarget(pSource.GetName(), pTarget.GetName())
	return 0	
	
def DamageSystemRandomly(pSystem, fMinPercent, fMaxPercent):
	if (pSystem):
		r = App.g_kSystemWrapper.GetRandomNumber(100)
		r = r / 100.0
		r = r * (fMaxPercent - fMinPercent)
		r = r + fMinPercent
		pSystem.SetConditionPercentage (r)
	
def DamageShipRandomly(pShip, fMinPercent, fMaxPercent):
	if (pShip):
		DamageSystemRandomly(pShip.GetHull(), fMinPercent, fMaxPercent)
		DamageSystemRandomly(pShip.GetShields(), fMinPercent, fMaxPercent)
		DamageSystemRandomly(pShip.GetPowerSubsystem(), fMinPercent, fMaxPercent)
		DamageSystemRandomly(pShip.GetSensorSubsystem(), fMinPercent, fMaxPercent)
		DamageSystemRandomly(pShip.GetImpulseEngineSubsystem(), fMinPercent, fMaxPercent)
		DamageSystemRandomly(pShip.GetWarpEngineSubsystem(), fMinPercent, fMaxPercent)
		DamageSystemRandomly(pShip.GetTorpedoSystem(), fMinPercent, fMaxPercent)
		DamageSystemRandomly(pShip.GetPhaserSystem(), fMinPercent, fMaxPercent)
		DamageSystemRandomly(pShip.GetPulseWeaponSystem(), fMinPercent, fMaxPercent)
		DamageSystemRandomly(pShip.GetTractorBeamSystem(), fMinPercent, fMaxPercent)
		DamageSystemRandomly(pShip.GetRepairSubsystem(), fMinPercent, fMaxPercent)
		DamageSystemRandomly(pShip.GetCloakingSubsystem(), fMinPercent, fMaxPercent)
		
		
		