# Engineering script Override for QuickBattle and QuickBattle Replacement by Defiant <mail@defiant.homedns.org>

import App
import MissionLib
import Foundation

# Load QBautostart Plugins
import LoadQBautostart


# QuickBattle
# ===========

# Get some needed Vars vor QuickBattle
import QuickBattle.QuickBattle
ET_PRELOAD_DONE = QuickBattle.QuickBattle.ET_PRELOAD_DONE
g_kEnemyList = QuickBattle.QuickBattle.g_kEnemyList
g_kFriendList = QuickBattle.QuickBattle.g_kFriendList

# QuickBattle.QuickBattle.SetupEventHandlers()
def QBSetupEventHandlers():

	pGame = App.Game_GetCurrentGame()

	pEpisode = pGame.GetCurrentEpisode()

	pMission = pEpisode.GetCurrentMission()



	App.g_kEventManager.AddBroadcastPythonFuncHandler(App.ET_OBJECT_DESTROYED, pMission, __name__ + ".QBShipDestroyed")

	App.g_kEventManager.AddBroadcastPythonFuncHandler(App.ET_OBJECT_EXPLODING, pMission, __name__ + ".QBShipExploding")

	App.g_kEventManager.AddBroadcastPythonFuncHandler(ET_PRELOAD_DONE, pMission, __name__ + ".QBStartSimulation2")

	# Init and Start
	LoadQBautostart.ImportQBautostart(0,1)


# QuickBattle.QuickBattle.StartSimulationAction(pAction)
def QBStartSimulationAction(pAction):

	kShipList = g_kEnemyList + g_kFriendList

	# Restart
	LoadQBautostart.ImportQBautostart(1,1)

	for kShipData in kShipList:

		# Get the name of the ship module, from the event type.

		# sShipType, sShipName, sDestroyedMessage, sWhichAI, sWhichSide, sAINumber = kShipData

		sShipRef, sAINumber, sDestroyedMessage, sWhichAI, sWhichSide = kShipData

		sShipType = sShipRef.shipFile

		sShipName = sShipRef.name



		# Import the ship module.

		pModule = __import__("ships." + sShipType)

		pModule.PreLoadModel ()



	# Add a preload done event

	pGame = App.Game_GetCurrentGame()

	pEpisode = pGame.GetCurrentEpisode()

	pMission = pEpisode.GetCurrentMission()

	pEvent = App.TGEvent_Create ()

	pEvent.SetEventType (ET_PRELOAD_DONE)

	pEvent.SetDestination (pMission)

	pGame.SetPreLoadDoneEvent (pEvent)



	return 0


def QBShipDestroyed(pObject, pEvent):
	QuickBattle.QuickBattle.ShipDestroyed(pObject, pEvent)


def QBShipExploding(pObject, pEvent):
	QuickBattle.QuickBattle.ShipExploding(pObject, pEvent)


def QBStartSimulation2(pObject, pEvent):
	QuickBattle.QuickBattle.StartSimulation2(pObject, pEvent)


def QBRecreatePlayer():

	qbGameMode = Foundation.BuildGameMode()
	qbGameMode.Activate()

	g_sPlayerType = qbGameMode.startShipDef

	g_sBridgeType = qbGameMode.GetBridge()

	g_pSet = QuickBattle.QuickBattle.g_pSet
	pPlayer = MissionLib.GetPlayer()

	pSet = None

	if pPlayer:

		pSet = pPlayer.GetContainingSet()

		pPlayer.SetDead()

		pSet.DeleteObjectFromSet(pPlayer.GetName())

	if not pSet:

		pSet = g_pSet


	pPlayer = MissionLib.CreatePlayerShip(g_sPlayerType.shipFile, pSet, "Player", "")


	# If there was a previous Action and the Players first ship still exists
	# return here to prevent a failing
	if (pPlayer == None):
		pPlayer = MissionLib.GetPlayer()
		pPlayer.DestroySystem(pPlayer.GetHull())
		return pPlayer
		

	# Reposition the ship...

	vLocation = pPlayer.GetWorldLocation()

	vForward = pPlayer.GetWorldForwardTG()



	# Make sure the ship's location won't overlap any other objects in the world.

	kPoint = App.TGPoint3()

	QuickBattle.QuickBattle.ChooseNewLocation(vLocation, vForward)

	kPoint.Set(vLocation)

	kPoint.Add(vForward)



	fRadius = pPlayer.GetRadius () * 2.0



	while pSet.IsLocationEmptyTG(kPoint, fRadius, 1) == 0:

		QuickBattle.QuickBattle.ChooseNewLocation(vLocation, vForward)

		kPoint.Set(vLocation)

		kPoint.Add(vForward)



	pPlayer.SetTranslate(kPoint)



	# Update the ship with its new positional information...

	pPlayer.UpdateNodeOnly()



	# update the proximity manager with this object's new position.

	pProximityManager = g_pSet.GetProximityManager()

	if (pProximityManager):

		pProximityManager.UpdateObject (pPlayer)



	# Remove plasmas.

	pTorpSys = pPlayer.GetTorpedoSystem()

	if(pTorpSys):

		# Forcibly unload all torps past quantums (Type 2)

		iNumTypes = pTorpSys.GetNumAmmoTypes()

		for iType in range(iNumTypes):

			if (iType >= 8):

				pTorpSys.RemoveAmmoType (iType);



	# Reload our bridge, just in case

	import LoadBridge

	LoadBridge.Load(g_sBridgeType)



	# Reassign global pointers

	QuickBattle.QuickBattle.AssignGlobalPointers()



	return pPlayer


# QuickBattle Replacement
# =======================

# Get some needed Vars vor QuickBattle Replacement
import Bridge.BridgeUtils
import Lib.Ambiguity
import Custom.QuickBattleGame.QuickBattle
import Custom.QuickBattleGame.QBGUI


# Custom.QuickBattleGame.QuickBattle.SetupEventHandlers(pMission)
def QBRSetupEventHandlers(pMission):
	App.g_kEventManager.AddBroadcastPythonFuncHandler(App.ET_OBJECT_DESTROYED, pMission, __name__ + ".QBRShipDestroyed")
	App.g_kEventManager.AddBroadcastPythonFuncHandler(App.ET_OBJECT_EXPLODING, pMission, __name__ + ".QBRShipExploding")

	pKiskaMenu = Bridge.BridgeUtils.GetBridgeMenu("Helm")
	if (pKiskaMenu != None):
		pKiskaMenu.AddPythonFuncHandlerForInstance(App.ET_DOCK,	__name__ + ".QBRDockButtonClicked")
		pKiskaMenu.AddPythonFuncHandlerForInstance(App.ET_HAIL,	__name__ + ".QBRHailHandler")

	pBrexMenu = Bridge.BridgeUtils.GetBridgeMenu("Engineer")
	pBridge = App.g_kSetManager.GetSet('bridge')
	g_pBrex = App.CharacterClass_GetObject(pBridge, "Engineer")
	if (pBrexMenu != None):
		event = Lib.Ambiguity.getEvent("ET_TRANSPORT")
		g_pBrex.AddPythonFuncHandlerForInstance(event, __name__ + ".QBRTransport")
		pButton = Custom.QuickBattleGame.QuickBattle.CreateBridgeMenuButton(App.TGString('Transporter'), event, 0, g_pBrex)
		pBrexMenu.PrependChild(pButton)

	
	event = Lib.Ambiguity.addEventHandler("XO", "ET_CONFIG", __name__ + ".QBRConfig")
	Lib.Ambiguity.createMenuButton("XO", "Configure", event, "Configure")
	event = Lib.Ambiguity.addEventHandler("XO", "ET_CREDITS", __name__ + ".QBRCredits")
	Lib.Ambiguity.createMenuButton("XO", "Credits", event, "Credits")

	# Init and Start
	LoadQBautostart.ImportQBautostart(0,0)


# Custom.QuickBattleGame.QuickBattle.StartSimulation(pObject, pEvent)
def QBRStartSimulation(pObject, pEvent):
	if (Custom.QuickBattleGame.QBGUI.CheckSetup() == 0):
		pTopWindow = App.TopWindow_GetTopWindow()
		pModalDialogWindow = App.ModalDialogWindow_Cast (pTopWindow.FindMainWindow (App.MWT_MODAL_DIALOG))
		pModalDialogWindow.Run(None, App.TGString("Not enough data. Please check that all your ships have a name and a starting system and that you have assigned a PLAYER ship."), App.TGString("OK"), None, None, None)
	else:
		Custom.QuickBattleGame.QuickBattle.ResetQB()
		if (pObject):
			pObject.CallNextHandler(pEvent)
	# Restart
	LoadQBautostart.ImportQBautostart(1,0)


def QBRConfig(pObject, pEvent):
	Custom.QuickBattleGame.QuickBattle.Config(pObject, pEvent)


def QBRCredits(pObject, pEvent):
	Custom.QuickBattleGame.QuickBattle.Credits(pObject, pEvent)


def QBRTransport(pObject, pEvent):
	Custom.QuickBattleGame.QuickBattle.Transport(pObject, pEvent)


def QBRHailHandler(TGObject, pEvent):
	Custom.QuickBattleGame.QuickBattle.HailHandler(TGObject, pEvent)


def QBRDockButtonClicked(pObject, pEvent):	
	Custom.QuickBattleGame.QuickBattle.DockButtonClicked(pObject, pEvent)


def QBRShipExploding(pObject, pEvent):
	Custom.QuickBattleGame.QuickBattle.ShipExploding(pObject, pEvent)


def QBRShipDestroyed(pObject, pEvent):
	Custom.QuickBattleGame.QuickBattle.ShipDestroyed(pObject, pEvent)
